@@ -29,6 +29,7 @@ import (
 	"golang.org/x/tools/go/packages/packagestest"
 	"golang.org/x/tools/gopls/internal/lsp/command"
 	"golang.org/x/tools/gopls/internal/lsp/protocol"
+	"golang.org/x/tools/gopls/internal/lsp/safetoken"
 	"golang.org/x/tools/gopls/internal/lsp/source"
 	"golang.org/x/tools/gopls/internal/lsp/source/completion"
 	"golang.org/x/tools/gopls/internal/lsp/tests/compare"
@@ -92,7 +93,7 @@ type WorkspaceSymbols = map[WorkspaceSymbolsTestType]map[span.URI][]string
 type Signatures = map[span.Span]*protocol.SignatureHelp
 type Links = map[span.URI][]Link
 type AddImport = map[span.URI]string
-type Hovers = map[span.Span]string
+type SelectionRanges = []span.Span
 
 type Data struct {
 	Config                   packages.Config
@@ -127,7 +128,7 @@ type Data struct {
 	Signatures               Signatures
 	Links                    Links
 	AddImport                AddImport
-	Hovers                   Hovers
+	SelectionRanges          SelectionRanges
 
 	fragments map[string]string
 	dir       string
@@ -137,15 +138,15 @@ type Data struct {
 	ModfileFlagAvailable bool
 
 	mappersMu sync.Mutex
-	mappers   map[span.URI]*protocol.ColumnMapper
+	mappers   map[span.URI]*protocol.Mapper
 }
 
-// TODO(adonovan): there are multiple implementations of this (undocumented)
-// interface, each of which must implement similar semantics. For example:
-// - *runner in ../cmd/test/check.go
-// - *runner in ../source/source_test.go
-// - *runner in ../lsp_test.go
-// Can we avoid this duplication?
+// The Tests interface abstracts the LSP-based implementation of the marker
+// test operators (such as @codelens) appearing in files beneath ../testdata/.
+//
+// TODO(adonovan): reduce duplication; see https://github.com/golang/go/issues/54845.
+// There is only one implementation (*runner in ../lsp_test.go), so
+// we can abolish the interface now.
 type Tests interface {
 	CallHierarchy(*testing.T, span.Span, *CallHierarchyResult)
 	CodeLens(*testing.T, span.URI, []protocol.CodeLens)
@@ -176,7 +177,7 @@ type Tests interface {
 	SignatureHelp(*testing.T, span.Span, *protocol.SignatureHelp)
 	Link(*testing.T, span.URI, []Link)
 	AddImport(*testing.T, span.URI, string)
-	Hover(*testing.T, span.Span, string)
+	SelectionRanges(*testing.T, span.Span)
 }
 
 type Definition struct {
@@ -282,7 +283,6 @@ func DefaultOptions(o *source.Options) {
 	o.InsertTextFormat = protocol.SnippetTextFormat
 	o.CompletionBudget = time.Minute
 	o.HierarchicalDocumentSymbolSupport = true
-	o.ExperimentalWorkspaceModule = true
 	o.SemanticTokens = true
 	o.InternalOptions.NewDiff = "both"
 }
@@ -295,10 +295,6 @@ func RunTests(t *testing.T, dataDir string, includeMultiModule bool, f func(*tes
 	}
 	for _, mode := range modes {
 		t.Run(mode, func(t *testing.T) {
-			if mode == "MultiModule" {
-				// Some bug in 1.12 breaks reading markers, and it's not worth figuring out.
-				testenv.NeedsGo1Point(t, 13)
-			}
 			datum := load(t, mode, dataDir)
 			t.Helper()
 			f(t, datum)
@@ -333,13 +329,12 @@ func load(t testing.TB, mode string, dir string) *Data {
 		Signatures:               make(Signatures),
 		Links:                    make(Links),
 		AddImport:                make(AddImport),
-		Hovers:                   make(Hovers),
 
 		dir:       dir,
 		fragments: map[string]string{},
 		golden:    map[string]*Golden{},
 		mode:      mode,
-		mappers:   map[span.URI]*protocol.ColumnMapper{},
+		mappers:   map[span.URI]*protocol.Mapper{},
 	}
 
 	if !*UpdateGolden {
@@ -484,7 +479,6 @@ func load(t testing.TB, mode string, dir string) *Data {
 		"implementations": datum.collectImplementations,
 		"typdef":          datum.collectTypeDefinitions,
 		"hoverdef":        datum.collectHoverDefinitions,
-		"hover":           datum.collectHovers,
 		"highlight":       datum.collectHighlights,
 		"inlayHint":       datum.collectInlayHints,
 		"refs":            datum.collectReferences,
@@ -499,6 +493,7 @@ func load(t testing.TB, mode string, dir string) *Data {
 		"incomingcalls":   datum.collectIncomingCalls,
 		"outgoingcalls":   datum.collectOutgoingCalls,
 		"addimport":       datum.collectAddImports,
+		"selectionrange":  datum.collectSelectionRanges,
 	}); err != nil {
 		t.Fatal(err)
 	}
@@ -573,9 +568,6 @@ func Run(t *testing.T, tests Tests, data *Data) {
 					if strings.Contains(t.Name(), "cgo") {
 						testenv.NeedsTool(t, "cgo")
 					}
-					if strings.Contains(t.Name(), "declarecgo") {
-						testenv.NeedsGo1Point(t, 15)
-					}
 					test(t, src, e, data.CompletionItems)
 				})
 			}
@@ -760,9 +752,6 @@ func Run(t *testing.T, tests Tests, data *Data) {
 				if strings.Contains(t.Name(), "cgo") {
 					testenv.NeedsTool(t, "cgo")
 				}
-				if strings.Contains(t.Name(), "declarecgo") {
-					testenv.NeedsGo1Point(t, 15)
-				}
 				tests.Definition(t, spn, d)
 			})
 		}
@@ -788,16 +777,6 @@ func Run(t *testing.T, tests Tests, data *Data) {
 		}
 	})
 
-	t.Run("Hover", func(t *testing.T) {
-		t.Helper()
-		for pos, info := range data.Hovers {
-			t.Run(SpanName(pos), func(t *testing.T) {
-				t.Helper()
-				tests.Hover(t, pos, info)
-			})
-		}
-	})
-
 	t.Run("InlayHints", func(t *testing.T) {
 		t.Helper()
 		for _, src := range data.InlayHints {
@@ -947,6 +926,15 @@ func Run(t *testing.T, tests Tests, data *Data) {
 		}
 	})
 
+	t.Run("SelectionRanges", func(t *testing.T) {
+		t.Helper()
+		for _, span := range data.SelectionRanges {
+			t.Run(SpanName(span), func(t *testing.T) {
+				tests.SelectionRanges(t, span)
+			})
+		}
+	})
+
 	if *UpdateGolden {
 		for _, golden := range data.golden {
 			if !golden.Modified {
@@ -1039,6 +1027,7 @@ func checkData(t *testing.T, data *Data) {
 	fmt.Fprintf(buf, "SignaturesCount = %v\n", len(data.Signatures))
 	fmt.Fprintf(buf, "LinksCount = %v\n", linksCount)
 	fmt.Fprintf(buf, "ImplementationsCount = %v\n", len(data.Implementations))
+	fmt.Fprintf(buf, "SelectionRangesCount = %v\n", len(data.SelectionRanges))
 
 	want := string(data.Golden(t, "summary", summaryFile, func() ([]byte, error) {
 		return buf.Bytes(), nil
@@ -1052,7 +1041,7 @@ func checkData(t *testing.T, data *Data) {
 	}
 }
 
-func (data *Data) Mapper(uri span.URI) (*protocol.ColumnMapper, error) {
+func (data *Data) Mapper(uri span.URI) (*protocol.Mapper, error) {
 	data.mappersMu.Lock()
 	defer data.mappersMu.Unlock()
 
@@ -1061,7 +1050,7 @@ func (data *Data) Mapper(uri span.URI) (*protocol.ColumnMapper, error) {
 		if err != nil {
 			return nil, err
 		}
-		data.mappers[uri] = protocol.NewColumnMapper(uri, content)
+		data.mappers[uri] = protocol.NewMapper(uri, content)
 	}
 	return data.mappers[uri], nil
 }
@@ -1240,6 +1229,10 @@ func (data *Data) collectDefinitions(src, target span.Span) {
 	}
 }
 
+func (data *Data) collectSelectionRanges(spn span.Span) {
+	data.SelectionRanges = append(data.SelectionRanges, spn)
+}
+
 func (data *Data) collectImplementations(src span.Span, targets []span.Span) {
 	data.Implementations[src] = targets
 }
@@ -1286,10 +1279,6 @@ func (data *Data) collectHoverDefinitions(src, target span.Span) {
 	}
 }
 
-func (data *Data) collectHovers(src span.Span, expected string) {
-	data.Hovers[src] = expected
-}
-
 func (data *Data) collectTypeDefinitions(src, target span.Span) {
 	data.Definitions[src] = Definition{
 		Src:    src,
@@ -1348,7 +1337,7 @@ func (data *Data) collectSymbols(name string, selectionRng span.Span, kind, deta
 // mustRange converts spn into a protocol.Range, panicking on any error.
 func (data *Data) mustRange(spn span.Span) protocol.Range {
 	m, err := data.Mapper(spn.URI())
-	rng, err := m.Range(spn)
+	rng, err := m.SpanRange(spn)
 	if err != nil {
 		panic(fmt.Sprintf("converting span %s to range: %v", spn, err))
 	}
@@ -1360,7 +1349,7 @@ func (data *Data) collectWorkspaceSymbols(typ WorkspaceSymbolsTestType) func(*ex
 		if data.WorkspaceSymbols[typ] == nil {
 			data.WorkspaceSymbols[typ] = make(map[span.URI][]string)
 		}
-		pos := data.Exported.ExpectFileSet.Position(note.Pos)
+		pos := safetoken.StartPosition(data.Exported.ExpectFileSet, note.Pos)
 		uri := span.URIFromPath(pos.Filename)
 		data.WorkspaceSymbols[typ][uri] = append(data.WorkspaceSymbols[typ][uri], query)
 	}
@@ -1390,7 +1379,7 @@ func (data *Data) collectCompletionSnippets(spn span.Span, item token.Pos, plain
 }
 
 func (data *Data) collectLinks(spn span.Span, link string, note *expect.Note, fset *token.FileSet) {
-	position := fset.Position(note.Pos)
+	position := safetoken.StartPosition(fset, note.Pos)
 	uri := spn.URI()
 	data.Links[uri] = append(data.Links[uri], Link{
 		Src:          spn,
